# -*- coding: utf-8 -*-

"""
MLConjug.

A Python library to conjugate verbs of in French, English, Spanish, Italian, Portuguese and Romanian (mores soon) using Machine Learning techniques.
Any verb in one of the supported language can be conjugated as the module contains a Machine Learning model of how the verbs behave.
Even completely new or made-up verbs can be successfully conjugated in this manner.
The supplied pre-trained models are composed of:

- a binary feature extractor,
- a feature selector using Linear Support Vector Classification,
- a classifier using Stochastic Gradient Descent.

MLConjug uses scikit-learn to implement the Machine Learning algorithms.
Users of the library can use any compatible classifiers from scikit-learn to modify and retrain the model.

Usage example:
    $ mlconjug manger

    $ mlconjug bring -l en

    $ mlconjug gallofar --language es
"""

__author__ = """SekouD"""
__email__ = 'sekoud.python@gmail.com'
__version__ = '2.1.1'

from .mlconjug import EndingCountVectorizer, DataSet, Model, LinearSVC, SGDClassifier, SelectFromModel, precision_recall_fscore_support, Conjugator, Pipeline
from .PyVerbiste import Verbiste



# -*- coding: utf-8 -*-

"""
PyVerbiste.

A Python library for conjugating verbs in French, English, Spanish, Italian, Portuguese and Romanian (mores soon).
It contains conjugation data generated by machine learning models using the python library mlconjug.
More information about mlconjug at https://pypi.org/project/mlconjug/


The conjugation data conforms to the XML schema defined by Verbiste.
More information on Verbiste at https://perso.b2b2c.ca/~sarrazip/dev/verbiste.html

"""

__author__ = 'SekouD'
__author_email__ = 'sekoud.python@gmail.com'

import copy
import xml.etree.ElementTree as ET
import codecs
from collections import OrderedDict
import pkg_resources
from _io import BufferedReader

RESOURCE_PACKAGE = __name__

LANGUAGES = ('default', 'fr', 'en', 'es', 'it', 'pt', 'ro')

VERBS_RESOURCE_PATH = {'fr': '/'.join(('data', 'verbiste', 'verbs-fr.xml')),
                       'it': '/'.join(('data', 'verbiste', 'verbs-it.xml')),
                       'es': '/'.join(('data', 'verbiste', 'verbs-es.xml')),
                       'en': '/'.join(('data', 'verbiste', 'verbs-en.xml')),
                       'pt': '/'.join(('data', 'verbiste', 'verbs-pt.xml')),
                       'ro': '/'.join(('data', 'verbiste', 'verbs-ro.xml')),}

CONJUGATIONS_RESOURCE_PATH = {'fr': '/'.join(('data', 'verbiste', 'conjugation-fr.xml')),
                              'it': '/'.join(('data', 'verbiste', 'conjugation-it.xml')),
                              'es': '/'.join(('data', 'verbiste', 'conjugation-es.xml')),
                              'en': '/'.join(('data', 'verbiste', 'conjugation-en.xml')),
                              'pt': '/'.join(('data', 'verbiste', 'conjugation-pt.xml')),
                              'ro': '/'.join(('data', 'verbiste', 'conjugation-ro.xml')),}

ABBREVS = ("1s", "2s", "3s", "1p", "2p", "3p")

PRONOUNS = {'fr': {'abbrev': ABBREVS,
                   'pronoun': ("je", "tu", "il (elle, on)", "nous", "vous", "ils (elles)")},
            'it': {'abbrev': ABBREVS,
                   'pronoun': ('io', 'tu', 'egli/ella', 'noi', 'voi', 'essi/esse')},
            'es': {'abbrev': ABBREVS,
                   'pronoun': ('yo', 'tú', 'él', 'nosotros', 'vosotros', 'ellos')},
            'en': {'abbrev': ABBREVS,
                   'pronoun': ('I', 'you', 'he/she/it', 'you', 'we', 'they')},
            'pt': {'abbrev': ABBREVS,
                   'pronoun': ('eu', 'tu', 'ele', 'nós', 'vós', 'eles')},
            'ro': {'abbrev': ABBREVS,
                   'pronoun': ('eu', 'tu', 'el/ea', 'noi', 'voi', 'ei/ele')}}

IMPERATIVE_PRONOUNS = {'fr': {'abbrev': ("2s :", "1p :", "2p :"),
                              'pronoun': ("tu", "nous", "vous")},
                       'it': None,
                       'es': {'abbrev': ("2s :", "3s :", "1p :", "2p :", "3p :"),
                              'pronoun': ('tú', 'él', 'nosotros', 'vosotros', 'ellos')},
                       'en': {'abbrev': ("2s :", "1p :", "2p :"),
                              'pronoun': ("", "let's", "")},
                       'pt': None,
                       'ro': None}

GENDER = {'fr': {'abbrev': ("ms :", "mp :", "fs :", "fp :"),
                 'pronoun': ("masculin singulier", "masculin pluriel", "feminin singulier", "feminin pluriel")},
          'it': None,
          'es': None,
          'en': None,
          'pt': None,
          'ro': None}

NEGATION = {'fr': 'ne',
            'it': 'non',
            'es': 'no',
            'en': "don't",
            'pt': 'não',
            'ro': 'nu'}


class Verbiste:
    """
    This is the class handling the Verbiste xml files.

    :param language: string.
        The language of the conjugator. The default value is fr for French.
        The allowed values are: fr, en, es, it, pt, ro

    """

    def __init__(self, language='default'):
        if language not in LANGUAGES:
            raise ValueError('Unsupported language.\nThe allowed languages are fr, en, es, it, pt, ro')
        if language == 'default':
            self.language = 'fr'
        else:
            self.language = language
        self.verbs = {}
        self.conjugations = OrderedDict()
        verbs_file = pkg_resources.resource_stream(RESOURCE_PACKAGE, VERBS_RESOURCE_PATH[self.language])
        self._load_verbs(verbs_file)
        conjugations_file = pkg_resources.resource_stream(RESOURCE_PACKAGE, CONJUGATIONS_RESOURCE_PATH[self.language])
        self._load_conjugations(conjugations_file)
        self.templates = sorted(self.conjugations.keys())
        self.model = None

    def _load_verbs(self, verbs_file):
        """
        Load and parses the verbs from xml file.

        :param verbs_file: string or path object.
            Path to the verbs xml file.
        """
        if isinstance(verbs_file, BufferedReader):
            verbs_dic = self._parse_verbs(verbs_file)
        else:
            with codecs.open(verbs_file, "r", encoding='utf-8') as file:
                verbs_dic = self._parse_verbs(file)
        self.verbs = verbs_dic

    def _parse_verbs(self, file):
        """
        Parses XML file

        :param file: FileObject.
            XML file containing the verbs.
        :return: OrderedDict.
            An OrderedDict containing the verb and its template for all verbs in the file.
        """
        verbs_dic = {}
        xml = ET.parse(file)
        for verb in xml.findall("v"):
            verb_name = verb.find("i").text
            template = verb.find("t").text
            index = - len(template[template.index(":") + 1:])
            root = verb_name[:index]
            verbs_dic[verb_name] = {"template": template, "root": root}
        return verbs_dic

    def _load_conjugations(self, conjugations_file):
        """
        Load and parses the conjugations from xml file.

        :param conjugations_file: string or path object.
            Path to the conjugation xml file.
        """
        if isinstance(conjugations_file, BufferedReader):
            conjugations_dic = self._parse_conjugations(conjugations_file)
        else:
            with codecs.open(conjugations_file, "r", encoding='utf-8') as file:
                conjugations_dic = self._parse_conjugations(file)
        self.conjugations = conjugations_dic

    def _parse_conjugations(self, file):
        """
        Parses XML file

        :param file: FileObject.
            XML file containing the conjugation templates
        :return: OrderedDict
            An OrderedDict containing all the conjugation templates in the file.
        """
        conjugations_dic = {}
        xml = ET.parse(file)
        for template in xml.findall("template"):
            template_name = template.get("name")
            conjugations_dic[template_name] = OrderedDict()
            for mood in list(template):
                conjugations_dic[template_name][mood.tag] = OrderedDict()
                for tense in list(mood):
                    conjugations_dic[template_name][mood.tag][tense.tag] = self._load_tense(tense)
        return conjugations_dic

    def _load_tense(self, tense):
        """

        :param tense: string.
            The current tense being processed.
        :return: list.
            List of conjugated suffixes.
        """
        persons = list(tense)
        if not persons:
            conjug = None
        elif len(persons) == 1:
            if persons[0].find("i") is not None:
                conjug = persons[0].find("i").text
            else:
                conjug = None
        else:
            conjug = [(pers, term.find("i").text if term.find("i") is not None else None) for pers, term in
                      enumerate(persons)]
        return conjug

    def get_verb_info(self, verb):
        """
        Gets verb information and returns a VerbInfo instance.

        :param verb: string.
            Verb to conjugate.
        :return: VerbInfo object or None.
        """
        if verb not in self.verbs.keys():
            return None
        else:
            infinitive = verb
            root = self.verbs[verb]['root']
            template = self.verbs[verb]['template']
            verb_info = VerbInfo(infinitive, root, template)
            return verb_info

    def get_conjug_info(self, template):
        """
        Gets conjugation information corresponding to the given template.

        :param template: string.
            Name of the verb ending pattern.
        :return: OrderedDict or None.
        """
        if template not in self.conjugations.keys():
            return None
        else:
            info = copy.deepcopy(self.conjugations[template])
            return info


class VerbInfo(object):
    """
    This class defines the Verbiste verb information structure.

    :param infinitive: string.
        Infinitive form of the verb.
    :param root: string.
        Lexical root of the verb.
    :param template: string.
        Name of the verb ending pattern.

    """
    __slots__ = ('infinitive', 'root', 'template')

    def __init__(self, infinitive, root, template):
        self.infinitive = infinitive
        self.root = root
        self.template = template

    def __eq__(self, other):
        if self.infinitive == other.infinitive and self.root == other.root and self.root == other.root:
            return True
        else:
            return False


class Verb(object):
    """
    This class defines the Verb Object.

    :param verb_info: VerbInfo Object.
    :param conjug_info: OrderedDict.
    :param subject: srting.
        Toggles abbreviated or full pronouns.
        The default value is 'abbrev'.
        Select 'pronoun' for full pronouns.

    """

    def __init__(self, verb_info, conjug_info, subject='abbrev'):
        self.name = verb_info.infinitive
        self.verb_info = verb_info
        self.conjug_info = conjug_info
        self.subject = subject
        self._load_conjug()
        self.language = 'default'

    def _load_conjug(self):
        """
        Populates the inflected forms of the verb.
        This is the generic version of this method.
        It does not add personal pronouns to the conjugated forms.

        """
        for mood, tense in self.conjug_info.items():
            for tense_name, persons in tense.items():
                if isinstance(persons, list):
                    persons_dict = OrderedDict()
                    for pers, term in persons:
                        if len(persons) == 6:
                            key = ABBREVS[pers]
                        else:
                            key = pers
                        if term is not None:
                            persons_dict[key] = self.verb_info.root + term
                        else:
                            persons[key] = None
                    self.conjug_info[mood][tense_name] = persons_dict
                elif isinstance(persons, str):
                    self.conjug_info[mood][tense_name] = self.verb_info.root + persons
        return


class VerbFr(Verb):
    """
    This class defines the French Verb Object.

    """

    language = 'fr'

    def _load_conjug(self):
        """
        Populates the inflected forms of the verb.

        """
        for mood, tense in self.conjug_info.items():
            for tense_name, persons in tense.items():
                if isinstance(persons, list):
                    persons_dict = OrderedDict()
                    for pers, term in persons:
                        if len(persons) == 6:
                            key = PRONOUNS[self.language][self.subject][pers]
                        elif tense_name == 'past-participle':
                            key = GENDER[self.language][self.subject][pers]
                        elif tense_name == 'imperative-present':
                            key = IMPERATIVE_PRONOUNS[self.language][self.subject][pers]
                        else:
                            key = term
                        if term is not None:
                            persons_dict[key] = self.verb_info.root + term
                        else:
                            persons_dict[key] = None
                    self.conjug_info[mood][tense_name] = persons_dict
                elif isinstance(persons, str):
                    self.conjug_info[mood][tense_name] = self.verb_info.root + persons
        return


class VerbEn(Verb):
    """
    This class defines the English Verb Object.

    """

    language = 'en'

    def _load_conjug(self):
        """
        Populates the inflected forms of the verb.

        """
        for mood, tense in self.conjug_info.items():
            for tense_name, persons in tense.items():
                if isinstance(persons, list):
                    persons_dict = OrderedDict()
                    for pers, term in persons:
                        if len(persons) == 6:
                            key = PRONOUNS[self.language][self.subject][pers]
                        elif tense_name == 'infinitive-present':
                            key = 'to'
                        elif tense_name == 'imperative-present':
                            key = IMPERATIVE_PRONOUNS[self.language][self.subject][pers]
                        else:
                            key = term
                        if term is not None:
                            persons_dict[key] = self.verb_info.root + term
                        else:
                            persons_dict[key] = None
                    self.conjug_info[mood][tense_name] = persons_dict
                elif isinstance(persons, str):
                    if tense_name == 'infinitive-present':
                        prefix = 'to '
                    else:
                        prefix = ''
                    self.conjug_info[mood][tense_name] = prefix + self.verb_info.root + persons
        return


class VerbEs(Verb):
    """
    This class defines the Spanish Verb Object.

    """

    language = 'es'

    def _load_conjug(self):
        """
        Populates the inflected forms of the verb.

        """
        for mood, tense in self.conjug_info.items():
            for tense_name, persons in tense.items():
                if isinstance(persons, list):
                    persons_dict = OrderedDict()
                    for pers, term in persons:
                        if len(persons) == 6:
                            key = PRONOUNS[self.language][self.subject][pers]
                        elif tense_name == 'Imperativo':
                            key = IMPERATIVE_PRONOUNS[self.language][self.subject][pers]
                        elif tense_name == 'Imperativo-non':
                            key = IMPERATIVE_PRONOUNS[self.language][self.subject][pers] + ' ' + NEGATION[self.language]
                        else:
                            key = pers
                        if term is not None:
                            persons_dict[key] = self.verb_info.root + term
                        else:
                            persons_dict[key] = None
                    self.conjug_info[mood][tense_name] = persons_dict
                elif isinstance(persons, str):
                    self.conjug_info[mood][tense_name] = self.verb_info.root + persons
        return


class VerbIt(Verb):
    """
    This class defines the Italian Verb Object.

    """

    language = 'it'

    def _load_conjug(self):
        """
        Populates the inflected forms of the verb.

        """
        for mood, tense in self.conjug_info.items():
            for tense_name, persons in tense.items():
                if isinstance(persons, list):
                    persons_dict = OrderedDict()
                    for pers, term in persons:
                        if len(persons) == 6 and not tense_name.startswith('Imperativo'):
                            key = PRONOUNS[self.language][self.subject][pers]
                        elif tense_name == 'Imperativo':
                            key = pers
                        elif tense_name == 'Imperativo-non':
                            key = NEGATION[self.language]
                        else:
                            key = pers
                        if term is not None:
                            persons_dict[key] = self.verb_info.root + term
                        else:
                            persons_dict[key] = None
                    self.conjug_info[mood][tense_name] = persons_dict
                elif isinstance(persons, str):
                    self.conjug_info[mood][tense_name] = self.verb_info.root + persons
        return


class VerbPt(Verb):
    """
    This class defines the Portuguese Verb Object.

    """

    language = 'pt'

    def _load_conjug(self):
        """
        Populates the inflected forms of the verb.

        """
        for mood, tense in self.conjug_info.items():
            for tense_name, persons in tense.items():
                if isinstance(persons, list):
                    persons_dict = OrderedDict()
                    for pers, term in persons:
                        if len(persons) == 6 and not tense_name.startswith('Imperativo'):
                            key = PRONOUNS[self.language][self.subject][pers]
                        elif tense_name == 'Imperativo-Afirmativo':
                            key = pers
                        elif tense_name == 'Imperativo-Negativo':
                            key = NEGATION[self.language]
                        else:
                            key = pers
                        if term is not None:
                            persons_dict[key] = self.verb_info.root + term
                        else:
                            persons_dict[key] = None
                    self.conjug_info[mood][tense_name] = persons_dict
                elif isinstance(persons, str):
                    self.conjug_info[mood][tense_name] = self.verb_info.root + persons
        return


class VerbRo(Verb):
    """
    This class defines the Romanian Verb Object.

    """

    language = 'ro'

    def _load_conjug(self):
        """
        Populates the inflected forms of the verb.

        """
        for mood, tense in self.conjug_info.items():
            for tense_name, persons in tense.items():
                if isinstance(persons, list):
                    persons_dict = OrderedDict()
                    for pers, term in persons:
                        if len(persons) == 6:
                            key = PRONOUNS[self.language][self.subject][pers]
                        elif tense_name == 'Imperativ':
                            key = pers
                        elif tense_name == 'Imperativ-Negativ':
                            key = NEGATION[self.language]
                        else:
                            key = pers
                        if term is not None:
                            persons_dict[key] = self.verb_info.root + term
                        else:
                            persons_dict[key] = None
                    self.conjug_info[mood][tense_name] = persons_dict
                elif isinstance(persons, str):
                    if tense_name == 'Infinitiv':
                        prefix = 'a '
                    else:
                        prefix = ''
                    self.conjug_info[mood][tense_name] = prefix + self.verb_info.root + persons
        return


if __name__ == "__main__":
    pass



# -*- coding: utf-8 -*-

"""Main module."""

from .PyVerbiste import Verbiste, Verb, VerbFr, VerbEn, VerbEs, VerbIt, VerbPt, VerbRo, VerbInfo

from sklearn.feature_selection import SelectFromModel
from sklearn.feature_extraction.text import CountVectorizer
from sklearn.svm import LinearSVC
from sklearn.linear_model import SGDClassifier
from sklearn.pipeline import Pipeline
from sklearn.metrics import precision_recall_fscore_support

import random
from collections import defaultdict
import pickle

import pkg_resources

RESOURCE_PACKAGE = __name__

LANGUAGES = ('fr', 'en', 'es', 'it', 'pt', 'ro')

VERBS = {'fr': VerbFr,
         'en': VerbEn,
         'es': VerbEs,
         'it': VerbIt,
         'pt': VerbPt,
         'ro': VerbRo}

PRE_TRAINED_MODEL_PATH = {'fr': '/'.join(('data', 'models', 'trained_model-fr.pickle')),
                          'it': '/'.join(('data', 'models', 'trained_model-it.pickle')),
                          'es': '/'.join(('data', 'models', 'trained_model-es.pickle')),
                          'en': '/'.join(('data', 'models', 'trained_model-en.pickle')),
                          'pt': '/'.join(('data', 'models', 'trained_model-pt.pickle')),
                          'ro': '/'.join(('data', 'models', 'trained_model-ro.pickle'))}


class Conjugator:
    """
    This is the main class of the project.
    The class manages the Verbiste data set and provides an interface with the scikit-learn model.
    If no parameters are provided The default language is set to french and the pre trained french conjugation model is used.
    It defines the method conjugate(verb, language) which is the main method of the module.

    :param language:
        Language of the conjugator. The default language is 'fr' for french.
    :param model:
        A user provided model if the user has trained his own model.

    """
    def __init__(self, language='fr', model=None):
        self.language = language
        self.verbiste = Verbiste(language=language)
        self.data_set = DataSet(self.verbiste)
        self.data_set.split_data(proportion=0.9)
        if not model:
            model = pickle.loads(pkg_resources.resource_stream(
                RESOURCE_PACKAGE, PRE_TRAINED_MODEL_PATH[language]).read())
        self.model = model

    def conjugate(self, verb, subject='abbrev'):
        """
        This is the main method of this class.
        It first checks to see if the verb is in Verbiste. If it is not, and a pre-trained scikit-learn model has been supplied,
        the method then calls the model to predict the conjugation class of the provided verb.
        Returns a Verb object or None.

        :param verb: string.
            Verb to conjugate.
        :param subject: srting.
            Toggles abbreviated or full pronouns.
            The default value is 'abbrev'.
            Select 'pronoun' for full pronouns.
        :return verb_object: Verb object or None.
        """
        if verb not in self.verbiste.verbs.keys():
            if self.model is None:
                return None
            predicted = self.model.predict([verb])[0]
            template = self.verbiste.templates[predicted]
            index = - len(template[template.index(":") + 1:])
            root = verb[:index]
            verb_info = VerbInfo(verb, root, template)
            conjug_info = self.verbiste.get_conjug_info(verb_info.template)
        else:
            infinitive = verb
            verb_info = self.verbiste.get_verb_info(infinitive)
            if verb_info is None:
                return None
            conjug_info = self.verbiste.get_conjug_info(verb_info.template)
            if conjug_info is None:
                return None
        verb_object = VERBS[self.language](verb_info, conjug_info, subject)
        return verb_object

    def set_model(self, model):
        """
        Assigns the provided pre-trained scikit-learn model to be able to conjugate unknown verbs.

        :param model: scikit-learn Classifier or Pipeline.
        """
        self.model = model
        return



class EndingCountVectorizer(CountVectorizer):
    """
    Custom Vectorizer optimized for extracting verbs features.
    The Vectorizer subclasses sklearn.feature_extraction.text.CountVectorizer
    As in Indo-European languages verbs are inflected by adding a morphological suffix,
    the vectorizer extracts verb endings and produces a vector representation of the verb with binary features.
    The features are the verb ending ngrams. (ngram_range is set at class initialization).

    """
    def _char_ngrams(self, verb):
        """
        Parses a verb and returns the ending ngrams.

        :param verb: string.
            Verb to vectorize.
        :return: list.
            Final ngrams of the verb.
        """
        verb = self._white_spaces.sub(" ", verb)
        verb_len = len(verb)
        ngrams = []
        min_n, max_n = self.ngram_range
        for n in range(min_n, min(max_n + 1, verb_len + 1)):
            ngram = verb[-n:]
            ngrams.append(ngram)
        return ngrams



class DataSet:
    """
    This class holds and manages the data set.
    Defines helper functions for managing Machine Learning Tasks like constructing a training and testing set.

    :param VerbisteObj:
        Instance of a Verbiste object.

    """
    def __init__(self, VerbisteObj):
        self.verbiste = VerbisteObj
        self.verbes = self.verbiste.verbs.keys()
        self.templates = sorted(self.verbiste.conjugations.keys())
        self.liste_verbes = []
        self.liste_templates = []
        self.dict_conjug = []
        self.train_input = []
        self.train_labels = []
        self.test_input = []
        self.test_labels = []
        self.construct_dict_conjug()

    def construct_dict_conjug(self):
        """
        Populates the dictionary containing the conjugation templates.

        """
        conjug = defaultdict(list)
        for verbe, info_verbe in self.verbiste.verbs.items():
            self.liste_verbes.append(verbe)
            self.liste_templates.append(
                self.templates.index(info_verbe["template"]))
            conjug[info_verbe["template"]].append(verbe)
        self.dict_conjug = conjug
        return

    def split_data(self, threshold=8, proportion=0.5):
        """
        Splits the data into a training and a testing set.

        :param threshold: int.
            Minimum size of conjugation class to be split.
        :param proportion: float.
            Proportion of samples in the training set. Must be between 0 and 1.
        """
        if proportion <= 0 or proportion >= 1:
            raise ValueError('The split proportion must be between 0 and 1')
        self.min_threshold = threshold
        self.split_proportion = proportion
        train_set = []
        test_set = []
        for template, lverbes in self.dict_conjug.items():
            if len(lverbes) <= threshold:
                for verbe in lverbes:
                    train_set.append((verbe, template))
            else:
                index = round(len(lverbes) * proportion)
                for verbe in lverbes[:index]:
                    train_set.append((verbe, template))
                for verbe in lverbes[index:]:
                    test_set.append((verbe, template))
        random.shuffle(train_set)
        random.shuffle(test_set)
        self.train_input = [elmt[0] for elmt in train_set]
        self.train_labels = [self.templates.index(elmt[1]) for elmt in train_set]
        self.test_input = [elmt[0] for elmt in test_set]
        self.test_labels = [self.templates.index(elmt[1]) for elmt in test_set]
        return


class Model(object):
    """
    This class manages the scikit-learn model.
    The Pipeline includes a feature vectorizer, a feature selector and a classifier.
    If any of the vectorizer, feature selector or classifier is not supplied at instance declaration,
    the __init__ method will provide good default values that get more than 92% prediction accuracy.

    :param vectorizer: scikit-learn Vectorizer.
    :param feature_selector: scikit-learn Classifier with a fit_transform() method
    :param classifier: scikit-learn Classifier with a predict() method
    """
    def __init__(self, vectorizer=None, feature_selector=None, classifier=None):
        if not vectorizer:
            vectorizer = EndingCountVectorizer(analyzer="char", binary=True, ngram_range=(2, 7))
        if not feature_selector:
            feature_selector = SelectFromModel(LinearSVC(penalty='l1', max_iter=12000, dual=False, verbose=2))
        if not classifier:
            classifier = SGDClassifier(loss='log', penalty='elasticnet', l1_ratio=0.15, max_iter=4000, alpha=1e-5, random_state=42, verbose=2)
        self.model = Pipeline([('vectorizer', vectorizer),
                               ('feature_selector', feature_selector),
                               ('classifier', classifier)])

    def train(self, samples, labels):
        """
        Trains the model on the supplied samples and labels.

        :param samples: list.
            List of verbs.
        :param labels: list.
            List of verb templates.
        """
        self.model = self.model.fit(samples, labels)
        return

    def predict(self, verbs):
        """
        Predicts the conjugation class of the provided list of verbs.

        :param verbs: list.
            List of verbs.
        :return: list.
            List of predicted conjugation groups.
        """
        prediction = self.model.predict(verbs)
        return prediction


if __name__ == "__main__":
    pass
